import parted
import unittest

try:
    import mock
except ImportError:
    has_mock = False
else:
    has_mock = True

import blivet
from blivet.size import Size

@unittest.skipUnless(has_mock, "Python mock module not available.")
class DiskLabelTestCase(unittest.TestCase):
    def testGetAlignment(self):
        with mock.patch("blivet.formats.disklabel.DiskLabel.freshPartedDisk", None):
            dl = blivet.formats.disklabel.DiskLabel()

        dl._partedDisk = mock.Mock()
        dl._partedDevice = mock.Mock()
        dl._partedDevice.sectorSize = 512

        # 512 byte grain sze
        disklabel_alignment = parted.Alignment(grainSize=1, offset=0)
        dl._partedDisk.partitionAlignment = disklabel_alignment

        # 1 MiB grain size
        minimal_alignment = parted.Alignment(grainSize = 2048, offset=0)
        dl._partedDevice.minimumAlignment = minimal_alignment

        # 4 MiB grain size
        optimal_alignment = parted.Alignment(grainSize=8192, offset=0)
        dl._partedDevice.optimumAlignment = optimal_alignment

        # expected end alignments
        optimal_end_alignment = parted.Alignment(
                                        grainSize=optimal_alignment.grainSize,
                                        offset=-1)
        minimal_end_alignment = parted.Alignment(
                                        grainSize=minimal_alignment.grainSize,
                                        offset=-1)

        # make sure the private methods all return the expected values
        self.assertEqual(dl._getDiskLabelAlignment(), disklabel_alignment)
        self.assertEqual(dl._getMinimalAlignment(), minimal_alignment)
        self.assertEqual(dl._getOptimalAlignment(), optimal_alignment)

        # validate result when passing a start alignment to getEndAlignment
        self.assertEqual(dl.getEndAlignment(alignment=optimal_alignment),
                                            optimal_end_alignment)
        self.assertEqual(dl.getEndAlignment(alignment=minimal_alignment),
                         minimal_end_alignment)

        # by default we should return the optimal alignment
        self.assertEqual(dl.getAlignment(), optimal_alignment)
        self.assertEqual(dl.getEndAlignment(), optimal_end_alignment)

        # when passed a size smaller than the optimal io size we should return
        # the minimal alignment
        self.assertEqual(dl.getAlignment(size=Size("2 MiB")), minimal_alignment)
        self.assertEqual(dl.getEndAlignment(size=Size("2 MiB")),
                         minimal_end_alignment)

        # test the old deprecated properties' values
        self.assertEqual(dl.alignment, dl._getOptimalAlignment())
        self.assertEqual(dl.endAlignment, dl.getEndAlignment())



import unittest
try:
    import mock
except ImportError:
    has_mock = False
else:
    has_mock = True

import blivet
from pykickstart.constants import CLEARPART_TYPE_ALL, CLEARPART_TYPE_LINUX, CLEARPART_TYPE_NONE
from parted import PARTITION_NORMAL
from blivet.flags import flags

@unittest.skipUnless(has_mock, "Python mock module not available.")
class ClearPartTestCase(unittest.TestCase):
    def setUp(self):
        flags.testing = True

    def testShouldClear(self):
        """ Test the Blivet.shouldClear method. """
        b = blivet.Blivet()

        DiskDevice = blivet.devices.DiskDevice
        PartitionDevice = blivet.devices.PartitionDevice

        # sda is a disk with an existing disklabel containing two partitions
        sda = DiskDevice("sda", size=100000, exists=True)
        sda.format = blivet.formats.getFormat("disklabel", device=sda.path,
                                              exists=True)
        sda.format._partedDisk = mock.Mock()
        sda.format._partedDevice = mock.Mock()
        sda.format._partedDisk.configure_mock(partitions=[])
        b.devicetree._addDevice(sda)

        # sda1 is a partition containing an existing ext4 filesystem
        sda1 = PartitionDevice("sda1", size=500, exists=True,
                               parents=[sda])
        sda1._partedPartition = mock.Mock(**{'type': PARTITION_NORMAL,
                                             'getFlag.return_value': 0})
        sda1.format = blivet.formats.getFormat("ext4", mountpoint="/boot",
                                               device=sda1.path,
                                               exists=True)
        b.devicetree._addDevice(sda1)

        # sda2 is a partition containing an existing vfat filesystem
        sda2 = PartitionDevice("sda2", size=10000, exists=True,
                               parents=[sda])
        sda2._partedPartition = mock.Mock(**{'type': PARTITION_NORMAL,
                                             'getFlag.return_value': 0})
        sda2.format = blivet.formats.getFormat("vfat", mountpoint="/foo",
                                               device=sda2.path,
                                               exists=True)
        b.devicetree._addDevice(sda2)

        # sdb is an unpartitioned disk containing an xfs filesystem
        sdb = DiskDevice("sdb", size=100000, exists=True)
        sdb.format = blivet.formats.getFormat("xfs", device=sdb.path,
                                              exists=True)
        b.devicetree._addDevice(sdb)

        # sdc is an unformatted/uninitialized/empty disk
        sdc = DiskDevice("sdc", size=100000, exists=True)
        b.devicetree._addDevice(sdc)

        # sdd is a disk containing an existing disklabel with no partitions
        sdd = DiskDevice("sdd", size=100000, exists=True)
        sdd.format = blivet.formats.getFormat("disklabel", device=sdd.path,
                                              exists=True)
        b.devicetree._addDevice(sdd)

        #
        # clearpart type none
        #
        b.config.clearPartType = CLEARPART_TYPE_NONE
        self.assertFalse(b.shouldClear(sda1),
                         msg="type none should not clear any partitions")
        self.assertFalse(b.shouldClear(sda2),
                         msg="type none should not clear any partitions")

        b.config.initializeDisks = False
        self.assertFalse(b.shouldClear(sda),
                         msg="type none should not clear non-empty disks")
        self.assertFalse(b.shouldClear(sdb),
                         msg="type none should not clear formatting from "
                             "unpartitioned disks")

        self.assertFalse(b.shouldClear(sdc),
                         msg="type none should not clear empty disk without "
                             "initlabel")
        self.assertFalse(b.shouldClear(sdd),
                         msg="type none should not clear empty partition table "
                             "without initlabel")

        b.config.initializeDisks = True
        self.assertFalse(b.shouldClear(sda),
                         msg="type none should not clear non-empty disks even "
                             "with initlabel")
        self.assertFalse(b.shouldClear(sdb),
                         msg="type non should not clear formatting from "
                             "unpartitioned disks even with initlabel")
        self.assertTrue(b.shouldClear(sdc),
                        msg="type none should clear empty disks when initlabel "
                            "is set")
        self.assertTrue(b.shouldClear(sdd),
                        msg="type none should clear empty partition table when "
                            "initlabel is set")

        #
        # clearpart type linux
        #
        b.config.clearPartType = CLEARPART_TYPE_LINUX
        self.assertTrue(b.shouldClear(sda1),
                        msg="type linux should clear partitions containing "
                            "ext4 filesystems")
        self.assertFalse(b.shouldClear(sda2),
                         msg="type linux should not clear partitions "
                             "containing vfat filesystems")

        b.config.initializeDisks = False
        self.assertFalse(b.shouldClear(sda),
                         msg="type linux should not clear non-empty disklabels")
        self.assertTrue(b.shouldClear(sdb),
                        msg="type linux should clear linux-native whole-disk "
                            "formatting regardless of initlabel setting")
        self.assertFalse(b.shouldClear(sdc),
                         msg="type linux should not clear unformatted disks "
                             "unless initlabel is set")
        self.assertFalse(b.shouldClear(sdd),
                         msg="type linux should not clear disks with empty "
                             "partition tables unless initlabel is set")

        b.config.initializeDisks = True
        self.assertFalse(b.shouldClear(sda),
                         msg="type linux should not clear non-empty disklabels")
        self.assertTrue(b.shouldClear(sdb),
                        msg="type linux should clear linux-native whole-disk "
                            "formatting regardless of initlabel setting")
        self.assertTrue(b.shouldClear(sdc),
                         msg="type linux should clear unformatted disks when "
                             "initlabel is set")
        self.assertTrue(b.shouldClear(sdd),
                         msg="type linux should clear disks with empty "
                             "partition tables when initlabel is set")

        sda1.protected = True
        self.assertFalse(b.shouldClear(sda1),
                         msg="protected devices should never be cleared")
        self.assertFalse(b.shouldClear(sda),
                         msg="disks containing protected devices should never "
                             "be cleared")
        sda1.protected = False

        #
        # clearpart type all
        #
        b.config.clearPartType = CLEARPART_TYPE_ALL
        self.assertTrue(b.shouldClear(sda1),
                        msg="type all should clear all partitions")
        self.assertTrue(b.shouldClear(sda2),
                        msg="type all should clear all partitions")

        b.config.initializeDisks = False
        self.assertTrue(b.shouldClear(sda),
                        msg="type all should initialize all disks")
        self.assertTrue(b.shouldClear(sdb),
                        msg="type all should initialize all disks")
        self.assertTrue(b.shouldClear(sdc),
                        msg="type all should initialize all disks")
        self.assertTrue(b.shouldClear(sdd),
                        msg="type all should initialize all disks")

        b.config.initializeDisks = True
        self.assertTrue(b.shouldClear(sda),
                        msg="type all should initialize all disks")
        self.assertTrue(b.shouldClear(sdb),
                        msg="type all should initialize all disks")
        self.assertTrue(b.shouldClear(sdc),
                        msg="type all should initialize all disks")
        self.assertTrue(b.shouldClear(sdd),
                        msg="type all should initialize all disks")

        sda1.protected = True
        self.assertFalse(b.shouldClear(sda1),
                         msg="protected devices should never be cleared")
        self.assertFalse(b.shouldClear(sda),
                         msg="disks containing protected devices should never "
                             "be cleared")
        sda1.protected = False

        #
        # clearpart type list
        #
        # TODO

    def tearDown(self):
        flags.testing = False

    def testInitializeDisk(self):
        """
            magic partitions
            non-empty partition table
        """
        pass

    @mock.patch.object(blivet.Blivet, "removeEmptyExtendedPartitions")
    @mock.patch.object(blivet.Blivet, "updateBootLoaderDiskList")
    def testClearPartitions(self, *args):
        """
            Disks reinitialization should be run based on shouldClear method.
            Under certain circumstances zerombr flag can override this behavior.
            This test checks these circumstances.
            The disks should be reinitialized when (and only when):
            * not marked as protected AND
            * zerombr flag is set OR shouldClear method returns True
            Note: When disk is marked as protected, shouldClear returns False
        """
        # pylint: disable=unused-argument
        b = blivet.Blivet()

        DiskDevice = blivet.devices.DiskDevice

        # sda is a disk with an existing disklabel containing two partitions
        sda = DiskDevice("sda", size=100000, exists=True)
        sda.format = blivet.formats.getFormat(None, device=sda.path,
                                              exists=True)
        sda.format._partedDisk = mock.Mock()
        sda.format._partedDevice = mock.Mock()
        sda.format._partedDisk.configure_mock(partitions=[])
        b.devicetree._addDevice(sda)

        sda.protected = False
        b.config.zeroMbr = False
        with mock.patch.object(blivet.Blivet, "shouldClear", return_value = False):
            with mock.patch.object(blivet.Blivet, "initializeDisk") as mock_initialize:
                b.clearPartitions()
                self.assertFalse(mock_initialize.called,
                                 "Trying to reinitialize a disk when shouldn't")

        sda.protected = False
        b.config.zeroMbr = False
        with mock.patch.object(blivet.Blivet, "shouldClear", return_value = True):
            with mock.patch.object(blivet.Blivet, "initializeDisk") as mock_initialize:
                b.clearPartitions()
                self.assertTrue(mock_initialize.called,
                                "Skipped disk reinitialization when shouldn't.")

        sda.protected = False
        b.config.zeroMbr = True
        with mock.patch.object(blivet.Blivet, "shouldClear", return_value = False):
            with mock.patch.object(blivet.Blivet, "initializeDisk") as mock_initialize:
                b.clearPartitions()
                self.assertTrue(mock_initialize.called,
                                "Skipped disk reinitialization when shouldn't.")

        sda.protected = False
        b.config.zeroMbr = True
        with mock.patch.object(blivet.Blivet, "shouldClear", return_value = True):
            with mock.patch.object(blivet.Blivet, "initializeDisk") as mock_initialize:
                b.clearPartitions()
                self.assertTrue(mock_initialize.called,
                                "Skipped disk reinitialization when shouldn't.")

        sda.protected = True
        b.config.zeroMbr = False
        with mock.patch.object(blivet.Blivet, "shouldClear", return_value = False):
            with mock.patch.object(blivet.Blivet, "initializeDisk") as mock_initialize:
                b.clearPartitions()
                self.assertFalse(mock_initialize.called,
                                 "Trying to reinitialize a disk when shouldn't")

        sda.protected = True
        b.config.zeroMbr = True
        with mock.patch.object(blivet.Blivet, "shouldClear", return_value = False):
            with mock.patch.object(blivet.Blivet, "initializeDisk") as mock_initialize:
                b.clearPartitions()
                self.assertFalse(mock_initialize.called,
                                 "Trying to reinitialize a disk when shouldn't")

    def testRecursiveRemove(self):
        """
            protected device at various points in stack
        """
        pass



#!/usr/bin/python

import abc

from tests import loopbackedtestcase
from blivet.errors import FSError

class LabelingAsRoot(loopbackedtestcase.LoopBackedTestCase):
    """Tests various aspects of labeling a filesystem where there
       is no easy way to read the filesystem's label once it has been
       set and where the filesystem can not be relabeled.
    """
    __metaclass__ = abc.ABCMeta

    _fs_class = abc.abstractproperty(
       doc="The class of the filesystem being tested on.")

    _invalid_label = abc.abstractproperty(
       doc="A label which is invalid for this filesystem.")

    def __init__(self, methodName='runTest'):
        super(LabelingAsRoot, self).__init__(methodName=methodName, deviceSpec=[102400])

    def setUp(self):
        an_fs = self._fs_class()
        if not an_fs.utilsAvailable:
            self.skipTest("utilities unavailable for filesystem %s" % an_fs.name)
        super(LabelingAsRoot, self).setUp()

    def testLabeling(self):
        """A sequence of tests of filesystem labeling.

           * create the filesystem when passing an invalid label
           * raise an exception when reading the filesystem
           * raise an exception when relabeling the filesystem
        """
        an_fs = self._fs_class(device=self.loopDevices[0], label=self._invalid_label)
        self.assertIsNone(an_fs.create())

        with self.assertRaisesRegexp(FSError, "no application to read label"):
            an_fs.readLabel()

        an_fs.label = "an fs"
        with self.assertRaisesRegexp(FSError, "no application to set label for filesystem"):
            an_fs.writeLabel()

    def testCreating(self):
        """Create the filesystem when passing a valid label """
        an_fs = self._fs_class(device=self.loopDevices[0], label="start")
        self.assertIsNone(an_fs.create())

    def testCreatingNone(self):
        """Create the filesystem when passing None
           (indicates filesystem default)
        """
        an_fs = self._fs_class(device=self.loopDevices[0], label=None)
        self.assertIsNone(an_fs.create())

    def testCreatingEmpty(self):
        """Create the filesystem when passing the empty label."""
        an_fs = self._fs_class(device=self.loopDevices[0], label="")
        self.assertIsNone(an_fs.create())

class LabelingWithRelabeling(LabelingAsRoot):
    """Tests labeling where it is possible to relabel.
    """

    def testLabeling(self):
        """A sequence of tests of filesystem labeling.

           * create the filesystem when passing an invalid label
           * raise an exception when reading the filesystem
           * relabel the filesystem with a valid label
           * relabel the filesystem with an empty label
           * raise an exception when relabeling when None is specified
           * raise an exception when relabeling with an invalid label
        """
        an_fs = self._fs_class(device=self.loopDevices[0], label=self._invalid_label)
        self.assertIsNone(an_fs.create())

        with self.assertRaisesRegexp(FSError, "no application to read label"):
            an_fs.readLabel()

        an_fs.label = "an fs"
        self.assertIsNone(an_fs.writeLabel())

        an_fs.label = ""
        self.assertIsNone(an_fs.writeLabel())

        an_fs.label = None
        with self.assertRaisesRegexp(FSError, "default label"):
            an_fs.writeLabel()

        an_fs.label = self._invalid_label
        with self.assertRaisesRegexp(FSError, "bad label format"):
            an_fs.writeLabel()

class CompleteLabelingAsRoot(LabelingAsRoot):
    """Tests where it is possible to read the label and to relabel
       an existing filesystem.
    """

    def testLabeling(self):
        """A sequence of tests of filesystem labeling.

           * create the filesystem when passing an invalid label
             and verify that the filesystem has the default label
           * relabel the filesystem with a valid label
             and verify that the filesystem has that label
           * relabel the filesystem with an empty label
             and verify that the filesystem has that label
           * raise an exception when relabeling when None is specified
           * raise an exception when relabeling with an invalid label
        """
        an_fs = self._fs_class(device=self.loopDevices[0], label=self._invalid_label)
        self.assertIsNone(an_fs.create())
        self.assertEqual(an_fs.readLabel(), an_fs._labelfs.default_label)

        an_fs.label = "an_fs"
        if an_fs.labelFormatOK(an_fs.label):
            self.assertIsNone(an_fs.writeLabel())
            self.assertEqual(an_fs.readLabel(), an_fs.label)
        else:
            self.assertRaisesRegexp(FSError, "bad label format", an_fs.writeLabel)

        an_fs.label = ""
        self.assertIsNone(an_fs.writeLabel())
        self.assertEqual(an_fs.readLabel(), an_fs.label)

        an_fs.label = None
        with self.assertRaisesRegexp(FSError, "default label"):
            an_fs.writeLabel()

        an_fs.label = "root___filesystem"
        with self.assertRaisesRegexp(FSError, "bad label format"):
            an_fs.writeLabel()

    def testCreating(self):
        """Create the filesystem when passing a valid label.
           Verify that the filesystem has that label.
        """
        an_fs = self._fs_class(device=self.loopDevices[0], label="start")
        self.assertIsNone(an_fs.create())
        if an_fs.labelFormatOK(an_fs.label):
            self.assertEqual(an_fs.readLabel(), "start")

    def testCreatingNone(self):
        """Create a filesystem with the label None.
           Verify that the filesystem has the default label.
        """
        an_fs = self._fs_class(device=self.loopDevices[0], label=None)
        self.assertIsNone(an_fs.create())
        self.assertEqual(an_fs.readLabel(), an_fs._labelfs.default_label)

    def testCreatingEmpty(self):
        """Create a filesystem with an empty label.
           Verify that the filesystem has the empty label.
        """
        an_fs = self._fs_class(device=self.loopDevices[0], label="")
        self.assertIsNone(an_fs.create())
        self.assertEqual(an_fs.readLabel(), "")



import unittest
try:
    from mock import patch
    from mock import PropertyMock
except ImportError:
    has_mock = False
else:
    has_mock = True
from pykickstart.version import returnClassForVersion
from blivet import Blivet
from blivet.devices import PartitionDevice
from blivet import formats
from blivet.size import Size


@unittest.skipUnless(has_mock, "Python mock module not available.")
class BlivetTestCase(unittest.TestCase):
    '''
    Define tests for the Blivet class
    '''
    def test_bootloader_in_kickstart(self):
        '''
        test that a bootloader such as prepboot/biosboot shows up
        in the kickstart data
        '''

        # prepboot test case
        with patch('blivet.Blivet.bootLoaderDevice', new_callable=PropertyMock) as mockBootLoaderDevice:
            with patch('blivet.Blivet.mountpoints', new_callable=PropertyMock) as mockMountpoints:
                # set up prepboot partition
                bootloader_device_obj = PartitionDevice("test_partition_device")
                bootloader_device_obj.size = Size('5MiB')
                bootloader_device_obj.format = formats.getFormat("prepboot")

                prepboot_blivet_obj = Blivet()

                # mountpoints must exist for updateKSData to run
                mockBootLoaderDevice.return_value = bootloader_device_obj
                mockMountpoints.values.return_value = []

                # initialize ksdata
                prepboot_ksdata = returnClassForVersion()()
                prepboot_blivet_obj.ksdata = prepboot_ksdata
                prepboot_blivet_obj.updateKSData()

        self.assertIn("part prepboot", str(prepboot_blivet_obj.ksdata))

        # biosboot test case
        with patch('blivet.Blivet.devices', new_callable=PropertyMock) as mockDevices:
            with patch('blivet.Blivet.mountpoints', new_callable=PropertyMock) as mockMountpoints:
                # set up biosboot partition
                biosboot_device_obj = PartitionDevice("biosboot_partition_device")
                biosboot_device_obj.size = Size('1MiB')
                biosboot_device_obj.format = formats.getFormat("biosboot")

                biosboot_blivet_obj = Blivet()

                # mountpoints must exist for updateKSData to run
                mockDevices.return_value = [biosboot_device_obj]
                mockMountpoints.values.return_value = []

                # initialize ksdata
                biosboot_ksdata = returnClassForVersion()()
                biosboot_blivet_obj.ksdata = biosboot_ksdata
                biosboot_blivet_obj.updateKSData()

        self.assertIn("part biosboot", str(biosboot_blivet_obj.ksdata))
